#include "TGQt.h"
#include <TROOT.h>
#include <TColor.h>
#include <TPoint.h>

#include <qglobal.h>
#include <qregion.h>
#include <qpointarray.h> 
#include <qwidget.h> 
#include <qmap.h>
#include <qpixmap.h>
#include <qpainter.h>

/*
#define TRACEGUI_TGQt() fprintf(stdout, "N/A GUI "\
		"TXQt::%s() %d\n", __FUNCTION__, __LINE__)*/
#define TRACEGUI_TGQt()

//#define TRACE_TGQt() fprintf(stdout, "TXQt::%s() %d\n", __FUNCTION__, __LINE__)
#define TRACE_TGQt()

//------------------------------------------------------------------------------
struct SVirtualWnd
{
	QPaintDevice *main;
	QPaintDevice *buffer;
	bool          isWidget;
};

class TGQtWndMan
{
protected:
	typedef QMap<int, SVirtualWnd> WindowMap;
	WindowMap fIdToWnd;

	SVirtualWnd  fDummy;
	SVirtualWnd& fCurrent;
	int          fCurrentId;
	QPainter     fPainter;

public:
	TGQtWndMan();
	virtual ~TGQtWndMan();

	int add(QWidget *w);
	int add(QPixmap *m);

	int removeById(int id);
	int removeByWinId(int winid);

	SVirtualWnd& getById(int id);
	SVirtualWnd& getByWinId(int winid);

	void select(int id);
	SVirtualWnd&  current() { return fCurrent; }
	int           currentId() { return fCurrentId; }
	QPaintDevice* currentDev();
	QPainter&     currentPainter() { return fPainter; }

	void setBuffer();
};

TGQtWndMan::TGQtWndMan()
	: fCurrent(fDummy)
{
	memset( &fDummy, 0, sizeof(SVirtualWnd) );
}

TGQtWndMan::~TGQtWndMan()
{
}

void
TGQtWndMan::setBuffer()
{
	if( !fCurrent.main ) return;
	if( !fCurrent.isWidget ) return;

	if( fCurrent.buffer )
	{
		fPainter.end();
		fPainter.begin( fCurrent.main );
		delete fCurrent.buffer;
		fCurrent.buffer = 0;
	}
	else
	{
		fPainter.end();
		QWidget* wg = (QWidget*) fCurrent.main;
		fCurrent.buffer = new QPixmap( wg->width(), wg->height() );
		fPainter.begin( fCurrent.buffer );
	}
}

void
TGQtWndMan::select(int id)
{
	QPaintDevice *p = currentDev();
	if( p ) fPainter.end();

	fCurrent = getById( id );
	p = currentDev();

	if( p ) fPainter.begin(p);
	fCurrentId = id;
}

int
TGQtWndMan::add(QWidget *w)
{
	size_t i;
	
	for(i=0; i<fIdToWnd.count(); i++)
		if( !fIdToWnd.contains(i+1) )
			break;

	i++;
	fIdToWnd[i].main = w;
	fIdToWnd[i].buffer = 0;
	fIdToWnd[i].isWidget = true;

	return i;
}

int
TGQtWndMan::add(QPixmap *m)
{
	size_t i;

	for(i=0; i<fIdToWnd.count(); i++)
		if( !fIdToWnd.contains(i+1) )
			break;

	i++;
	fIdToWnd[i].main = m;
	fIdToWnd[i].buffer = 0;
	fIdToWnd[i].isWidget = false;

	return i;
}

SVirtualWnd&
TGQtWndMan::getById(int id)
{
	if( fIdToWnd.contains(id) )
		return fIdToWnd[id];

	return fDummy;
}

SVirtualWnd&
TGQtWndMan::getByWinId(int winid)
{
	QWidget *w = QWidget::find( (WId)winid );

	if( !w )
		return fDummy;

	WindowMap::Iterator it;
	for ( it = fIdToWnd.begin(); it != fIdToWnd.end(); ++it )
		if( it.data().main == (QPaintDevice*) w ) {
			return fIdToWnd[it.key()];
		}

	return fDummy;
}

QPaintDevice*
TGQtWndMan::currentDev()
{
	if( fCurrent.buffer )
		return fCurrent.buffer;

	return fCurrent.main;
}

//------------------------------------------------------------------------------
class TGQtColorMan
{
protected:
	typedef QMap<int, QColor> ColorMap;
	ColorMap fColors;

public:
	TGQtColorMan();
	virtual ~TGQtColorMan();

	QColor& getColor(int id);
	void    putColor(int id, QColor color);
	QColor  getROOTColor(int id);
};

TGQtColorMan::TGQtColorMan()
{
}

TGQtColorMan::~TGQtColorMan()
{
}

QColor& TGQtColorMan::getColor(int id)
{
	if( fColors.contains(id) )
		return fColors[id];

	fColors[id] = getROOTColor( id );
	return fColors[id];
}

void TGQtColorMan::putColor(int id, QColor color)
{
	fColors[id] = color;
}

QColor TGQtColorMan::getROOTColor(int id)
{
	TColor *c = gROOT->GetColor(id);

	QColor qc(0, 0, 0);

	if( c )
		qc.setRgb( 
			(int)(c->GetRed()*255.0),
			(int)(c->GetGreen()*255.0),
			(int)(c->GetBlue()*255.0) );

	return qc;
}

//------------------------------------------------------------------------------
TGQt::TGQt()
{
	fOldX = gVirtualX;
	gVirtualX = this;

	fWndMan = new TGQtWndMan;
	fColorMan = new TGQtColorMan;

	fLinePen.setColor( QColor(0, 0, 0) );
	fLinePen.setWidth( 1 );
	fLinePen.setStyle( Qt::SolidLine );

	fFillBrush.setStyle( Qt::SolidPattern );
}

//------------------------------------------------------------------------------
TGQt::~TGQt()
{
	gVirtualX = fOldX;
}

//------------------------------------------------------------------------------
Bool_t 
TGQt::Init(
	void *display)
{
	TRACE_TGQt();
	ASSERT(0);
	return fOldX->Init(display);
}

//------------------------------------------------------------------------------
void
TGQt::ClearWindow()
{
	TRACE_TGQt();
}

//------------------------------------------------------------------------------
void
TGQt::ClosePixmap()
{
	TRACE_TGQt();
}

//------------------------------------------------------------------------------
void
TGQt::CloseWindow()
{
	TRACE_TGQt();
}

//------------------------------------------------------------------------------
void
TGQt::CopyPixmap(Int_t wid, Int_t xpos, Int_t ypos)
{
	SVirtualWnd& vw = fWndMan->getById( wid );

	QPaintDevice *src;
	if( vw.buffer ) src = vw.buffer;
	else src = vw.main;

	QPaintDevice *dest = fWndMan->currentDev();

	if( !dest || !src ) return;

	int width, height;
	if( vw.isWidget )
	{
		QWidget* t = (QWidget*) vw.main;
		width  = t->width();
		height = t->height();
	}
	else
	{
		QPixmap* t = (QPixmap*) vw.main;
		width  = t->width();
		height = t->height();
	}

	bitBlt(dest, xpos, ypos, src, 0, 0, width, height,
			Qt::CopyROP, FALSE);
}


//------------------------------------------------------------------------------
void
TGQt::CreateOpenGLContext(Int_t wid)
{
	TRACE_TGQt();
}


//------------------------------------------------------------------------------
void
TGQt::DeleteOpenGLContext(Int_t wid)
{
	TRACE_TGQt();
}

//------------------------------------------------------------------------------
void
TGQt::DrawBox(Int_t x1, Int_t y1, Int_t x2, Int_t y2, EBoxMode mode)
{
	QPainter &p = fWndMan->currentPainter();

	switch( mode )
	{
	case kHollow:
		p.setPen( fLinePen );
		p.drawRect( x1, y1, x2-x1, y2-y1 );
		break;

	case kFilled:
		p.setPen( QPen(Qt::NoPen) );
		p.fillRect( x1, y1, x2-x1, y2-y1, fFillBrush );
		break;
	}
}

//------------------------------------------------------------------------------
void
TGQt::DrawCellArray(Int_t x1, Int_t y1, Int_t x2, Int_t y2,
		Int_t nx, Int_t ny, Int_t *ic)
{
	TRACE_TGQt();
}

void
TGQt::DrawFillArea(Int_t n, TPoint *xy)
{
	QPointArray pa;

	pa.resize( n );

	for(int i=0; i<n; i++)
		pa.setPoint( i, xy[i].fX, xy[i].fY );

	QPainter &p = fWndMan->currentPainter();
	p.setBrush( fFillBrush );
	p.setPen( QPen( Qt::NoPen) );
	p.drawPolygon(pa);
}

void
TGQt::DrawLine(Int_t x1, Int_t y1, Int_t x2, Int_t y2)
{
	QPainter &p = fWndMan->currentPainter();
	p.setPen( fLinePen );
	p.drawLine( x1, y1, x2, y2 );
}

void
TGQt::DrawPolyLine(Int_t n, TPoint *xy)
{
	QPointArray pa;

	pa.resize( n );

	for(int i=0; i<n; i++)
		pa.setPoint( i, xy[i].fX, xy[i].fY );

	QPainter &p = fWndMan->currentPainter();
	p.setPen( fLinePen );
	p.drawPolyline(pa);
}


void
TGQt::DrawPolyMarker(Int_t n, TPoint *xy)
{
	QPointArray pa;

	pa.resize( n );

	for(int i=0; i<n; i++)
		pa.setPoint( i, xy[i].fX, xy[i].fY );

	QPainter &p = fWndMan->currentPainter();
	p.setPen( fLinePen );
	p.drawPoints(pa);
}


void
TGQt::DrawText(Int_t x, Int_t y, Float_t angle, Float_t mgn,
		const char *text, ETextMode mode)
{
	QPainter &p = fWndMan->currentPainter();
	p.setFont( fFont );
	QRect rc = p.fontMetrics().boundingRect(text);
	
	switch( fTextAlign%10 ) //vertical
	{
	case 1:
		y = y;
		break;
		
	case 2:
		y += rc.height()/2;
		break;
		
	case 3:
		y += rc.height();
		break;
	}

	switch( fTextAlign/10 ) //horizontal
	{
	case 1:
		x = x;
		break;
		
	case 2:
		x -= rc.width()/2;
		break;
		
	case 3:
		x -= rc.width();
		break;
	}

	
	p.drawText( x, y, text );
}

UInt_t
TGQt::ExecCommand(TGWin32Command *code)
{
	return fOldX->ExecCommand( code );
}


void
TGQt::GetCharacterUp(Float_t &chupx, Float_t &chupy)
{
	TRACE_TGQt();
}

Int_t
TGQt::GetDoubleBuffer(Int_t wid)
{
	TRACE_TGQt();
	return 0;
}


void
TGQt::GetGeometry(Int_t wid, Int_t &x, Int_t &y, UInt_t &w, UInt_t &h)
{
	SVirtualWnd& vw = fWndMan->getById(wid);
	if( !vw.main ) return;

	if( vw.isWidget )
	{
		QWidget *wg = (QWidget*) vw.main;
		x = wg->x();
		y = wg->y();
		w = wg->width();
		h = wg->height();
	}
	else
	{
		x = y = 0;
		QPixmap *px = (QPixmap*) vw.main;
		w = px->width();
		h = px->height();
	}
}

const char *
TGQt::DisplayName(const char *name)
{
	TRACE_TGQt();
	return 0;
}

Handle_t 
TGQt::GetNativeEvent() const
{
	TRACE_TGQt();
	return 0;
}


void
TGQt::GetPlanes(Int_t &nplanes)
{
	TRACE_TGQt();
}

void
TGQt::GetRGB(Int_t index, Float_t &r, Float_t &g, Float_t &b)
{
	QColor c = fColorMan->getColor( index );
	r = ((Float_t)c.red())/255.0;
	g = ((Float_t)c.green())/255.0;
	b = ((Float_t)c.blue())/255.0;
}


void
TGQt::GetTextExtent(UInt_t &w, UInt_t &h, char *mess)
{
	TRACE_TGQt();
}

Float_t
TGQt::GetTextMagnitude()
{
	return fTextMagnitude;
}

Window_t  
TGQt::GetWindowID(Int_t wid)
{
	TRACE_TGQt();
	return 0;
}

//------------------------------------------------------------------------------
Bool_t 
TGQt::HasTTFonts() const
{
	return true;
}

Int_t 
TGQt::InitWindow(ULong_t window)
{
	TRACE_TGQt();
	return 0;
}

//------------------------------------------------------------------------------
Int_t 
TGQt::AddWindow(ULong_t qwid, UInt_t w, UInt_t h)
{
	QWidget* wg = QWidget::find( (WId)qwid );

	if( wg )
		return fWndMan->add( wg );
	else // pixmap
		return fWndMan->add( (QPixmap*) qwid );

	return 0;
}

void
TGQt::RemoveWindow(ULong_t qwid)
{
	TRACE_TGQt();
}

void
TGQt::MoveWindow(Int_t wid, Int_t x, Int_t y)
{
	SVirtualWnd& w = fWndMan->getByWinId( wid );

	if( w.main && w.isWidget )
		((QWidget*)w.main)->move( x, y );
}

Int_t 
TGQt::OpenPixmap(UInt_t w, UInt_t h)
{
	QPixmap* px = new QPixmap( w, h );
	return fWndMan->add( px );
}

void
TGQt::QueryPointer(Int_t &ix, Int_t &iy)
{
	TRACE_TGQt();
}

void
TGQt::ReadGIF(Int_t x0, Int_t y0, const char *file)
{ 
	TRACE_TGQt();
}

Int_t 
TGQt::RequestLocator(Int_t mode, Int_t ctyp, Int_t &x, Int_t &y)
{
	TRACE_TGQt();
	return 0;
}

Int_t
TGQt::RequestString(Int_t x, Int_t y, char *text)
{
	TRACE_TGQt();
	return 0;
}

void
TGQt::RescaleWindow(Int_t wid, UInt_t w, UInt_t h)
{
	TRACE_TGQt();
}

Int_t
TGQt::ResizePixmap(Int_t wid, UInt_t w, UInt_t h)
{
	SVirtualWnd& vw = fWndMan->getById(wid);
	
	if( vw.isWidget ) 
		return 1;

	QPixmap *px = (QPixmap*) vw.main;

	if( wid == fWndMan->currentId() )
	{
		QPainter &p = fWndMan->currentPainter();
		p.end();
		px->resize( w, h );
		p.begin( px );
	}
	else
		px->resize( w, h );

	return 1;
}

void
TGQt::ResizeWindow(Int_t wid)
{
	return;
}

void
TGQt::SelectWindow(Int_t wid)
{
	fWndMan->select( wid );
}

void
TGQt::SetCharacterUp(Float_t chupx, Float_t chupy)
{
	TRACE_TGQt();
}

void
TGQt::SetClipOFF(Int_t wid)
{
	TRACE_TGQt();
}

void
TGQt::SetClipRegion(Int_t wid, Int_t x, Int_t y, UInt_t w, UInt_t h)
{
	TRACE_TGQt();
}

void
TGQt::SetCursor(Int_t win, ECursor cursor)
{
	TRACE_TGQt();
}

void
TGQt::SetDoubleBuffer(Int_t wid, Int_t mode)
{
	ASSERT( wid != 999 );

	int oldId = fWndMan->currentId();
	
	fWndMan->select(wid);

	if( !fWndMan->current().isWidget )
		return;

	if( mode )
		SetDoubleBufferON();
	else
		SetDoubleBufferOFF();

	fWndMan->select(oldId);
}

void
TGQt::SetDoubleBufferOFF()
{
	fWndMan->setBuffer();
}

void
TGQt::SetDoubleBufferON()
{
	fWndMan->setBuffer();
}

void
TGQt::SetDrawMode(EDrawMode mode)
{
	TRACE_TGQt();
}

void
TGQt::SetFillColor(Color_t cindex)
{
	fFillColor = cindex;
	fFillBrush.setColor( fColorMan->getColor(cindex) );
}

void
TGQt::SetFillStyle(Style_t style)
{
	TRACE_TGQt();
}

void
TGQt::SetLineColor(Color_t cindex)
{
	fLineColor = cindex;
	fLinePen.setColor( fColorMan->getColor(cindex) );
}

void
TGQt::SetLineType(Int_t n, Int_t *dash)
{
	TRACE_TGQt();
}

void
TGQt::SetLineStyle(Style_t linestyle)
{
	fLineStyle = linestyle;

	if( linestyle < 1 ) linestyle = 1;

	switch( linestyle )
	{
	case 1:
		fLinePen.setStyle( Qt::SolidLine );
		break;
		
	case 2:
		fLinePen.setStyle( Qt::DashLine );
		break;
		
	case 3:
		fLinePen.setStyle( Qt::DotLine );
		break;
		
	case 4:
		fLinePen.setStyle( Qt::DashDotLine );
		break;
	}
}

void
TGQt::SetLineWidth(Width_t width)
{
	fLineWidth = width;
	fLinePen.setWidth( width );
}

void
TGQt::SetMarkerColor(Color_t cindex)
{
	TRACE_TGQt();
}

void
TGQt::SetMarkerSize(Float_t markersize)
{
	TRACE_TGQt();
}

void
TGQt::SetMarkerStyle(Style_t markerstyle)
{
	TRACE_TGQt();
}

void
TGQt::SetOpacity(Int_t percent)
{
	TRACE_TGQt();
}

void
TGQt::SetRGB(Int_t cindex, Float_t r, Float_t g, Float_t b)
{
	fColorMan->putColor( cindex, 
			QColor( (int)(r*255.0), (int)(g*255.0), (int)(b*255.0) ) );
}

void
TGQt::SetTextAlign(Short_t talign)
{
	fTextAlign = talign;
}

void
TGQt::SetTextColor(Color_t cindex)
{
	TRACE_TGQt();
}

Int_t 
TGQt::SetTextFont(char *fontname, ETextSetMode mode)
{
	return 0;
}

void
TGQt::SetTextFont(Font_t fontnumber)
{
	TRACE_TGQt();
}

void
TGQt::SetTextMagnitude(Float_t mgn)
{
	fTextMagnitude = mgn;
}

void
TGQt::SetTextSize(Float_t textsize)
{
	fTextSize = textsize;
	fFont.setPointSizeFloat( textsize );
}

void
TGQt::UpdateWindow(Int_t mode)
{
	TRACE_TGQt();
}

void
TGQt::Warp(Int_t ix, Int_t iy)
{
	TRACE_TGQt();
}

void
TGQt::WriteGIF(char *name)
{
	TRACE_TGQt();
}

void
TGQt::WritePixmap(Int_t wid, UInt_t w, UInt_t h, char *pxname)
{
	TRACE_TGQt();
}




//--------------------------------- GUI ----------------------------------------	
void
TGQt::GetWindowAttributes(Window_t id, WindowAttributes_t &attr)
{
	TRACEGUI_TGQt();

	fOldX->GetWindowAttributes( id, attr );
}

void
TGQt::MapWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->MapWindow( id );
}

void
TGQt::MapSubwindows(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->MapSubwindows( id );
}

void
TGQt::MapRaised(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->MapRaised( id );
}

void
TGQt::UnmapWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->UnmapWindow( id );
}

void
TGQt::DestroyWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->DestroyWindow( id );
}

void
TGQt::RaiseWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->RaiseWindow( id );
}

void
TGQt::LowerWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->LowerWindow( id );
}

void
TGQt::MoveWindow(Window_t id, Int_t x, Int_t y)
{
	TRACEGUI_TGQt();

	fOldX->MoveWindow( id, x, y );
}

void
TGQt::MoveResizeWindow(Window_t id, Int_t x, Int_t y, UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->MoveResizeWindow( id, x, y, w, h );
}

void
TGQt::ResizeWindow(Window_t id, UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->ResizeWindow( id, w, h );
}

void
TGQt::IconifyWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->IconifyWindow( id );
}

void
TGQt::SetWindowBackground(Window_t id, ULong_t color)
{
	TRACEGUI_TGQt();

	fOldX->SetWindowBackground( id, color );
}

void
TGQt::SetWindowBackgroundPixmap(Window_t id, Pixmap_t pxm)
{
	TRACEGUI_TGQt();

	fOldX->SetWindowBackgroundPixmap( id, pxm );
}

Window_t 
TGQt::CreateWindow(Window_t parent, Int_t x, Int_t y,
	UInt_t w, UInt_t h, UInt_t border,
	Int_t depth, UInt_t clss, void *visual, SetWindowAttributes_t *attr,
	UInt_t wtype)
{
	TRACEGUI_TGQt();
	return fOldX->CreateWindow(parent, x, y, w, h, border, depth, clss, visual,
		attr, wtype);
}

Int_t 
TGQt::OpenDisplay(const char *dpyName)
{
	TRACEGUI_TGQt();

	return fOldX->OpenDisplay( dpyName );
}

void
TGQt::CloseDisplay()
{
	TRACEGUI_TGQt();

	fOldX->CloseDisplay();
}

Display_t 
TGQt::GetDisplay()
{
	TRACEGUI_TGQt();

	return fOldX->GetDisplay();
}

Atom_t
TGQt::InternAtom(const char *atom_name, Bool_t only_if_exist)
{
	TRACEGUI_TGQt();

	return fOldX->InternAtom( atom_name, only_if_exist );
}

Window_t 
TGQt::GetDefaultRootWindow()
{
	TRACEGUI_TGQt();

	return fOldX->GetDefaultRootWindow();
}

Window_t 
TGQt::GetParent(Window_t id)
{
	TRACEGUI_TGQt();

	return fOldX->GetParent( id );
}

FontStruct_t 
TGQt::LoadQueryFont(const char *font_name)
{
	TRACEGUI_TGQt();

	return fOldX->LoadQueryFont( font_name );
}

FontH_t 
TGQt::GetFontHandle(FontStruct_t fs)
{
	TRACEGUI_TGQt();

	return fOldX->GetFontHandle( fs );
}

void
TGQt::DeleteFont(FontStruct_t fs)
{
	TRACEGUI_TGQt();

	fOldX->DeleteFont( fs );
}

GContext_t 
TGQt::CreateGC(Drawable_t id, GCValues_t *gval)
{
	TRACEGUI_TGQt();

	return fOldX->CreateGC( id, gval );
}

void
TGQt::ChangeGC(GContext_t gc, GCValues_t *gval)
{
	TRACEGUI_TGQt();

	fOldX->ChangeGC( gc, gval );
}

void
TGQt::CopyGC(GContext_t org, GContext_t dest, Mask_t mask)
{
	TRACEGUI_TGQt();

	fOldX->CopyGC( org, dest, mask );
}

void
TGQt::DeleteGC(GContext_t gc)
{
	TRACEGUI_TGQt();

	fOldX->DeleteGC( gc );
}

Cursor_t
TGQt::CreateCursor(ECursor cursor)
{
	TRACEGUI_TGQt();

	return fOldX->CreateCursor( cursor );
}

void
TGQt::SetCursor(Window_t id, Cursor_t curid)
{
	TRACEGUI_TGQt();

	fOldX->SetCursor( id, curid );
}

Pixmap_t 
TGQt::CreatePixmap(Drawable_t id, UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	return fOldX->CreatePixmap( id, w, h );
}

Pixmap_t
TGQt::CreatePixmap(Drawable_t id, const char *bitmap, UInt_t width,
	UInt_t height, ULong_t forecolor, ULong_t backcolor,
	Int_t depth)
{
	TRACEGUI_TGQt();

	return fOldX->CreatePixmap( id, bitmap, width, height, 
		forecolor, backcolor, depth );
}

Pixmap_t
TGQt::CreateBitmap(Drawable_t id, const char *bitmap,
	UInt_t width, UInt_t height)
{
	TRACEGUI_TGQt();

	return fOldX->CreateBitmap( id, bitmap, width, height );
}

void
TGQt::DeletePixmap(Pixmap_t pmap)
{
	TRACEGUI_TGQt();

	fOldX->DeletePixmap( pmap );
}

Bool_t
TGQt::CreatePictureFromFile(Drawable_t id, const char *filename,
	Pixmap_t &pict, Pixmap_t &pict_mask,
	PictureAttributes_t &attr)
{
	TRACEGUI_TGQt();

	return fOldX->CreatePictureFromFile( id, filename, 
		pict, pict_mask, attr );
}

Bool_t 
TGQt::CreatePictureFromData(Drawable_t id, char **data,
	Pixmap_t &pict, Pixmap_t &pict_mask,
	PictureAttributes_t &attr)
{
	TRACEGUI_TGQt();

	return fOldX->CreatePictureFromData( id, data, 
		pict, pict_mask, attr );
}

Bool_t
TGQt::ReadPictureDataFromFile(const char *filename, char ***ret_data)
{
	TRACEGUI_TGQt();

	return fOldX->ReadPictureDataFromFile( filename, ret_data );
}

void
TGQt::DeletePictureData(void *data)
{
	TRACEGUI_TGQt();

	fOldX->DeletePictureData( data );
}

void
TGQt::SetDashes(GContext_t gc, Int_t offset, 
	const char *dash_list, Int_t n)
{
	TRACEGUI_TGQt();

	fOldX->SetDashes( gc, offset, dash_list, n );
}

Bool_t 
TGQt::ParseColor(Colormap_t cmap, const char *cname, ColorStruct_t &color)
{
	TRACEGUI_TGQt();

	return fOldX->ParseColor( cmap, cname, color );
}

Bool_t 
TGQt::AllocColor(Colormap_t cmap, ColorStruct_t &color)
{
	TRACEGUI_TGQt();

	return fOldX->AllocColor( cmap, color );
}

void
TGQt::QueryColor(Colormap_t cmap, ColorStruct_t &color)
{
	TRACEGUI_TGQt();

	fOldX->QueryColor( cmap, color );
}

Int_t 
TGQt::EventsPending()
{
	TRACEGUI_TGQt();

	return fOldX->EventsPending();
}

void
TGQt::NextEvent(Event_t &event)
{
	TRACEGUI_TGQt();

	fOldX->NextEvent( event );
}

void
TGQt::Bell(Int_t percent)
{
	TRACEGUI_TGQt();

	fOldX->Bell( percent );
}

void
TGQt::CopyArea(Drawable_t src, Drawable_t dest, GContext_t gc,
	Int_t src_x, Int_t src_y, UInt_t width,
	UInt_t height, Int_t dest_x, Int_t dest_y)
{
	TRACEGUI_TGQt();

	fOldX->CopyArea( src, dest, gc, src_x, src_y, width, height, 
		dest_x, dest_y );
}

void
TGQt::ChangeWindowAttributes(Window_t id, SetWindowAttributes_t *attr)
{
	TRACEGUI_TGQt();

	fOldX->ChangeWindowAttributes( id, attr );
}

void
TGQt::ChangeProperty(Window_t id, Atom_t property, Atom_t type,
	UChar_t *data, Int_t len)
{
	TRACEGUI_TGQt();

	fOldX->ChangeProperty( id, property, type, data, len );
}

void
TGQt::DrawLine(Drawable_t id, GContext_t gc, Int_t x1, Int_t y1, Int_t x2, Int_t y2)
{
	TRACEGUI_TGQt();

	fOldX->DrawLine( id, gc, x1, y1, x2, y2 );
}

void
TGQt::ClearArea(Window_t id, Int_t x, Int_t y, UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->ClearArea( id, x, y, w, h );
}

Bool_t 
TGQt::CheckEvent(Window_t id, EGEventType type, Event_t &ev)
{
	TRACEGUI_TGQt();

	return fOldX->CheckEvent( id, type, ev );
}

void
TGQt::SendEvent(Window_t id, Event_t *ev)
{
	TRACEGUI_TGQt();

	fOldX->SendEvent( id, ev );
}

void
TGQt::WMDeleteNotify(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->WMDeleteNotify( id );
}

void
TGQt::SetKeyAutoRepeat(Bool_t on)
{
	TRACEGUI_TGQt();

	fOldX->SetKeyAutoRepeat( on );
}

void
TGQt::GrabKey(Window_t id, Int_t keycode, UInt_t modifier, Bool_t grab)
{
	TRACEGUI_TGQt();

	fOldX->GrabKey( id, keycode, modifier, grab );
}

void
TGQt::GrabButton(Window_t id, EMouseButton button, UInt_t modifier,
	UInt_t evmask, Window_t confine, Cursor_t cursor,
	Bool_t grab)
{
	TRACEGUI_TGQt();

	fOldX->GrabButton( id, button, modifier, evmask, confine, cursor, grab );
}

void
TGQt::GrabPointer(Window_t id, UInt_t evmask, Window_t confine,
	Cursor_t cursor, Bool_t grab, Bool_t owner_events)
{
	TRACEGUI_TGQt();

	fOldX->GrabPointer( id, evmask, confine, cursor, grab, owner_events );
}

void
TGQt::SetWindowName(Window_t id, char *name)
{
	TRACEGUI_TGQt();

	fOldX->SetWindowName( id, name );
}

void
TGQt::SetIconName(Window_t id, char *name)
{
	TRACEGUI_TGQt();

	fOldX->SetIconName( id, name );
}

void
TGQt::SetIconPixmap(Window_t id, Pixmap_t pix)
{
	TRACEGUI_TGQt();

	fOldX->SetIconPixmap( id, pix );
}

void
TGQt::SetClassHints(Window_t id, char *className, char *resourceName)
{
	TRACEGUI_TGQt();

	fOldX->SetClassHints( id, className, resourceName );
}

void
TGQt::SetMWMHints(Window_t id, UInt_t value, UInt_t funcs, UInt_t input)
{
	TRACEGUI_TGQt();

	fOldX->SetMWMHints( id, value, funcs, input );
}

void
TGQt::SetWMPosition(Window_t id, Int_t x, Int_t y)
{
	TRACEGUI_TGQt();

	fOldX->SetWMPosition( id, x, y );
}

void
TGQt::SetWMSize(Window_t id, UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->SetWMSize( id, w, h );
}

void
TGQt::SetWMSizeHints(Window_t id, UInt_t wmin, UInt_t hmin,
	UInt_t wmax, UInt_t hmax, UInt_t winc, UInt_t hinc)
{
	TRACEGUI_TGQt();

	fOldX->SetWMSizeHints( id, wmin, hmin, wmax, hmax, winc, hinc );
}

void
TGQt::SetWMState(Window_t id, EInitialState state)
{
	TRACEGUI_TGQt();

	fOldX->SetWMState( id, state );
}

void
TGQt::SetWMTransientHint(Window_t id, Window_t main_id)
{
	TRACEGUI_TGQt();

	fOldX->SetWMTransientHint( id, main_id );
}

void
TGQt::DrawString(Drawable_t id, GContext_t gc, Int_t x, Int_t y,
	const char *s, Int_t len)
{
	TRACEGUI_TGQt();

	fOldX->DrawString( id, gc, x, y, s, len );
}

Int_t
TGQt::TextWidth(FontStruct_t font, const char *s, Int_t len)
{
	TRACEGUI_TGQt();

	return fOldX->TextWidth( font, s, len );
}

void
TGQt::GetFontProperties(FontStruct_t font, Int_t &max_ascent, Int_t &max_descent)
{
	TRACEGUI_TGQt();

	fOldX->GetFontProperties( font, max_ascent, max_descent );
}

void
TGQt::GetGCValues(GContext_t gc, GCValues_t &gval)
{
	TRACEGUI_TGQt();

	fOldX->GetGCValues( gc, gval );
}

FontStruct_t
TGQt::GetFontStruct(FontH_t fh)
{
	TRACEGUI_TGQt();

	return fOldX->GetFontStruct( fh );
}

void
TGQt::FreeFontStruct(FontStruct_t fs)
{
	TRACEGUI_TGQt();

	fOldX->FreeFontStruct( fs );
}

void
TGQt::ClearWindow(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->ClearWindow( id );
}

Int_t
TGQt::KeysymToKeycode(UInt_t keysym)
{
	TRACEGUI_TGQt();

	return fOldX->KeysymToKeycode( keysym );
}

void
TGQt::FillRectangle(Drawable_t id, GContext_t gc, Int_t x, Int_t y,
	UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->FillRectangle( id, gc, x, y, w, h );
}

void
TGQt::DrawRectangle(Drawable_t id, GContext_t gc, Int_t x, Int_t y,
	UInt_t w, UInt_t h)
{
	TRACEGUI_TGQt();

	fOldX->DrawRectangle( id, gc, x, y, w, h );
}

void
TGQt::DrawSegments(Drawable_t id, GContext_t gc, Segment_t *seg, Int_t nseg)
{
	TRACEGUI_TGQt();

	fOldX->DrawSegments( id, gc, seg, nseg );
}

void
TGQt::SelectInput(Window_t id, UInt_t evmask)
{
	TRACEGUI_TGQt();

	fOldX->SelectInput( id, evmask );
}

Window_t
TGQt::GetInputFocus()
{
	TRACEGUI_TGQt();

	return fOldX->GetInputFocus();
}

void
TGQt::SetInputFocus(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->SetInputFocus( id );
}

Window_t
TGQt::GetPrimarySelectionOwner()
{
	TRACEGUI_TGQt();

	return fOldX->GetPrimarySelectionOwner();
}

void
TGQt::SetPrimarySelectionOwner(Window_t id)
{
	TRACEGUI_TGQt();

	fOldX->SetPrimarySelectionOwner( id );
}

void
TGQt::ConvertPrimarySelection(Window_t id, Atom_t clipboard, Time_t when)
{
	TRACEGUI_TGQt();

	fOldX->ConvertPrimarySelection( id, clipboard, when );
}

void
TGQt::LookupString(Event_t *event, char *buf, Int_t buflen, UInt_t &keysym)
{
	TRACEGUI_TGQt();

	fOldX->LookupString( event, buf, buflen, keysym );
}

void
TGQt::GetPasteBuffer(Window_t id, Atom_t atom, TString &text, Int_t &nchar,
	Bool_t del)
{
	TRACEGUI_TGQt();

	fOldX->GetPasteBuffer( id, atom, text, nchar, del );
}

void
TGQt::TranslateCoordinates(Window_t src, Window_t dest, Int_t src_x,
	Int_t src_y, Int_t &dest_x, Int_t &dest_y, Window_t &child)
{
	TRACEGUI_TGQt();

	fOldX->TranslateCoordinates( src, dest, src_x, src_y,
		dest_x, dest_y, child );
}

void
TGQt::GetWindowSize(Drawable_t id, Int_t &x, Int_t &y, UInt_t &w, UInt_t &h)
{
	TRACEGUI_TGQt();

	fOldX->GetWindowSize( id, x, y, w, h );
}

void
TGQt::FillPolygon(Window_t id, GContext_t gc, Point_t *points, Int_t npnt)
{
	TRACEGUI_TGQt();

	fOldX->FillPolygon( id, gc, points, npnt );
}

void
TGQt::QueryPointer(Window_t id, Window_t &rootw, Window_t &childw,
	Int_t &root_x, Int_t &root_y, Int_t &win_x,
	Int_t &win_y, UInt_t &mask)
{
	TRACEGUI_TGQt();

	fOldX->QueryPointer( id, rootw, childw, root_x, root_y, win_x, win_y, mask );
}

void
TGQt::SetForeground(GContext_t gc, ULong_t foreground)
{
	TRACEGUI_TGQt();

	fOldX->SetForeground( gc, foreground );
}

void
TGQt::SetClipRectangles(GContext_t gc, Int_t x, Int_t y, Rectangle_t *recs, Int_t n)
{
	TRACEGUI_TGQt();

	fOldX->SetClipRectangles( gc, x, y, recs, n );
}

void
TGQt::Update(Int_t mode)
{
	TRACEGUI_TGQt();

	fOldX->Update( mode );
}

//------------------------------------------------------------------------------
//
// Region functions. Using QRegion instead of X-Window regions.
//

Region_t
TGQt::CreateRegion()
{
	QRegion *reg = new QRegion();
	return (Region_t) reg;
}

void
TGQt::DestroyRegion(Region_t reg)
{
	delete (QRegion*) reg;
}

void
TGQt::UnionRectWithRegion(Rectangle_t *rect, Region_t src, Region_t dest)
{
	if( !rect || src == 0 || dest == 0 )
		return;

	QRegion rc(QRect( rect->fX, rect->fY, rect->fWidth, rect->fHeight ));
	QRegion &rSrc  = *(QRegion*) src;
	QRegion &rDest = *(QRegion*) dest;

	rDest = rSrc + rc;
}

Region_t
TGQt::PolygonRegion(Point_t *points, Int_t np, Bool_t winding)
{
	if( np<0 || !points )
		return 0;

	QPointArray pa;
	pa.resize( np );
	for(int i=0; i<np; i++)
		pa.setPoint( i, points[i].fX, points[i].fY );

	return (Region_t) new QRegion( pa, winding );
}

void
TGQt::UnionRegion(Region_t rega, Region_t regb, Region_t result)
{
	if( !rega || !regb || !result )
		return;

	QRegion& a = *(QRegion*) rega;
	QRegion& b = *(QRegion*) regb;
	QRegion& r = *(QRegion*) result;

	r = a + b;
}

void
TGQt::IntersectRegion(Region_t rega, Region_t regb, Region_t result)
{
	if( !rega || !regb || !result )
		return;

	QRegion& a = *(QRegion*) rega;
	QRegion& b = *(QRegion*) regb;
	QRegion& r = *(QRegion*) result;

	r = a & b;
}

void
TGQt::SubtractRegion(Region_t rega, Region_t regb, Region_t result)
{
	if( !rega || !regb || !result )
		return;

	QRegion& a = *(QRegion*) rega;
	QRegion& b = *(QRegion*) regb;
	QRegion& r = *(QRegion*) result;

	r = a - b;
}

void
TGQt::XorRegion(Region_t rega, Region_t regb, Region_t result)
{
	if( !rega || !regb || !result )
		return;

	QRegion& a = *(QRegion*) rega;
	QRegion& b = *(QRegion*) regb;
	QRegion& r = *(QRegion*) result;

	r = a ^ b;
}

Bool_t
TGQt::EmptyRegion(Region_t reg)
{
	if( !reg )
		return true;

	QRegion& r = *(QRegion*) reg;

	return r.isEmpty();
}

Bool_t
TGQt::PointInRegion(Int_t x, Int_t y, Region_t reg)
{
	if( !reg )
		return false;

	QRegion& r = *(QRegion*) reg;
	return r.contains( QPoint(x, y) ); 
}

Bool_t
TGQt::EqualRegion(Region_t rega, Region_t regb)
{
	if( !rega || !regb )
		return false;

	QRegion& a = *(QRegion*) rega;
	QRegion& b = *(QRegion*) regb;

	return (a == b);
}

void
TGQt::GetRegionBox(Region_t reg, Rectangle_t *rect)
{
	if( !reg || !rect )
		return;

	QRegion& r = *(QRegion*) reg;
	QRect rc   = r.boundingRect();

	rect->fX      = rc.x();
	rect->fY      = rc.y();
	rect->fWidth  = rc.width();
	rect->fHeight = rc.height();
}

