#include <TApplication.h>
#include <TCanvas.h>
#include <TH1.h>
#include <TH2.h>
#include <TBox.h>
#include <TStyle.h>
#include <TPaveText.h>
#include <TF2.h>

#include <qapplication.h>
#include <qpushbutton.h>
#include <qpainter.h>
#include <qwidget.h>

#include "TGQt.h"
#include "CPxCanvas.h"

//------------------------------------------------------------------------------
class CMainWnd: public QWidget
{
protected:
	CPxCanvas *c1;
	TH1 *h1;
public:
	CMainWnd( QWidget* parent, const char* title );
	virtual ~CMainWnd();
	void resizeEvent(QResizeEvent *e);
	void mousePressEvent(QMouseEvent *e);
	void paintEvent(QPaintEvent *e);
};

CMainWnd::~CMainWnd()
{
	delete c1;
}

void
CMainWnd::mousePressEvent(QMouseEvent *e)
{
}

void
CMainWnd::paintEvent(QPaintEvent *e)
{
	QPainter p(this);
	p.drawPixmap( 0, 0, *c1 );
}

void 
CMainWnd::resizeEvent(QResizeEvent *e)
{
	QSize sz = e->size();
	c1->resize( sz.width(), sz.height() );
	c1->update();
}

CMainWnd::CMainWnd( QWidget* parent, const char* title )
	: QWidget( parent, title )
{
	c1 = new CPxCanvas("c1");
	
	c1->getCanvas()->SetFillColor(42);
	c1->getCanvas()->cd();
	gStyle->SetFrameFillColor(42);
	TPaveText *tit = new TPaveText(.2,0.96,.8,.995);
	tit->SetFillColor(33);
	tit->AddText("Examples of Surface options");
	tit->Draw();

	TPad *pad1 = new TPad("pad1","Gouraud shading",0.03,0.50,0.98,0.95,21);
	TPad *pad2 = new TPad("pad2","Color mesh",0.03,0.02,0.98,0.48,21);
	pad1->Draw();
	pad2->Draw();
	//
	// We generate a 2-D function
	TF2 *f2 = new TF2("f2","x**2 + y**2 - x**3 -8*x*y**4",-1,1.2,-1.5,1.5);
	f2->SetContour(48);
	f2->SetFillColor(45);

	// Draw this function in pad1 with Gouraud shading option
	pad1->cd();
	pad1->SetPhi(-80);
	pad1->SetLogz();
	f2->Draw("surf4");

	// Draw this function in pad2 with color mesh option
	pad2->cd();
	pad2->SetTheta(25);
	pad2->SetPhi(-110);
	pad2->SetLogz();
	f2->Draw("lego");

	c1->update();
}

//------------------------------------------------------------------------------
int 
main(int argc, char *argv[])
{
	QApplication qapp( argc, argv );
	TApplication app( "app", &argc, argv );

	TGQt vx;

	qapp.setGlobalMouseTracking(true);

	CMainWnd mainwnd( 0, "hi" );
	qapp.setMainWidget( &mainwnd );
	mainwnd.show();

	int res = qapp.exec();

	return res;
}
